@@ -32,11 +32,12 @@ import textwrap
 import pkg_resources
 import six
 
-from oslo_config._i18n import _LW
+
 from oslo_config import cfg
 import stevedore.named  # noqa
 
 LOG = logging.getLogger(__name__)
+UPPER_CASE_GROUP_NAMES = ['DEFAULT']
 
 _generator_opts = [
     cfg.StrOpt(
@@ -90,7 +91,8 @@ def _format_defaults(opt):
         elif opt.default is None:
             default_str = '<None>'
         elif (isinstance(opt, (cfg.StrOpt, cfg.IPOpt,
-                               cfg.HostnameOpt, cfg.URIOpt))):
+                               cfg.HostnameOpt, cfg.HostAddressOpt,
+                               cfg.URIOpt))):
             default_str = opt.default
         elif isinstance(opt, cfg.BoolOpt):
             default_str = str(opt.default).lower()
@@ -105,7 +107,7 @@ def _format_defaults(opt):
                                   key=operator.itemgetter(0))
             default_str = ','.join(['%s:%s' % i for i in sorted_items])
         else:
-            LOG.warning(_LW('Unknown option type: %s'), repr(opt))
+            LOG.warning('Unknown option type: %s', repr(opt))
             default_str = str(opt.default)
         defaults = [default_str]
 
@@ -205,7 +207,7 @@ class _OptFormatter(object):
         :returns: a formatted opt description string
         """
         if not opt.help:
-            LOG.warning(_LW('"%s" is missing a help string'), opt.dest)
+            LOG.warning('"%s" is missing a help string', opt.dest)
 
         opt_type = _format_type_name(opt.type)
         opt_prefix = ''
@@ -268,8 +270,12 @@ class _OptFormatter(object):
                               (opt.dest, err))
 
         for d in opt.deprecated_opts:
-            lines.append('# Deprecated group/name - [%s]/%s\n' %
-                         (d.group or group_name, d.name or opt.dest))
+            # NOTE(bnemec): opt names with a - are not valid in a config file,
+            # but it is possible to add a DeprecatedOpt with a - name.  We
+            # want to ignore those as they won't work anyway.
+            if d.name and '-' not in d.name:
+                lines.append('# Deprecated group/name - [%s]/%s\n' %
+                             (d.group or group_name, d.name or opt.dest))
 
         if opt.deprecated_for_removal:
             if opt.deprecated_since:
@@ -346,6 +352,22 @@ def _cleanup_opts(read_opts):
         if namespace not in clean:
             clean[namespace] = collections.OrderedDict()
         for group, opts in listing:
+            # NOTE: Normalize group names to lowe-case except those defined in
+            # UPPER_CASE_GROUP_NAMES
+            if group:
+                group_name = getattr(group, 'name', str(group))
+                if group_name.upper() in UPPER_CASE_GROUP_NAMES:
+                    normalized_gn = group_name.upper()
+                else:
+                    normalized_gn = group_name.lower()
+                if normalized_gn != group_name:
+                    LOG.warning('normalizing group name %r to %r', group_name,
+                                normalized_gn)
+                    if hasattr(group, 'name'):
+                        group.name = normalized_gn
+                    else:
+                        group = normalized_gn
+
             if group not in clean[namespace]:
                 clean[namespace][group] = collections.OrderedDict()
             for opt in opts:
@@ -354,8 +376,8 @@ def _cleanup_opts(read_opts):
     # recreate the list of (namespace, [(group, [opt_1, opt_2])]) tuples
     # from the cleaned structure.
     cleaned_opts = [
-        (namespace, [(group, list(clean[namespace][group].values()))
-                     for group in clean[namespace]])
+        (namespace, [(g, list(clean[namespace][g].values()))
+                     for g in clean[namespace]])
         for namespace in clean
     ]
 
